@@ -15,6 +15,7 @@ use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
+use DateTime::HiRes ();
 use DateTime::TimeZone ();
 use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
@@ -23,7 +24,7 @@ use Storable qw(dclone);
 
 use DateTime::Format::Natural::Utils qw(trim);
 
-our $VERSION = '1.13';
+our $VERSION = '1.18';
 
 validation_options(
     on_fail => sub
@@ -75,7 +76,7 @@ sub _init
     $self->{data} = $mod->__new();
     $self->{grammar_class} = $mod;
 
-    $self->{format_provided} = exists $opts{format};
+    $self->{mode} = '';
 }
 
 sub _init_check
@@ -141,6 +142,28 @@ sub _init_check
         daytime => {
             type => HASHREF,
             optional => true,
+            callbacks => {
+                'valid daytime' => sub
+                {
+                    my $href = shift;
+                    my %daytimes = map { $_ => true } qw(morning afternoon evening);
+                    if (any { !$daytimes{$_} } keys %$href) {
+                        die "spelling of daytime\n";
+                    }
+                    elsif (any { !defined $href->{$_} } keys %$href) {
+                        die "undefined hour\n";
+                    }
+                    elsif (any { $href->{$_} !~ /^\d{1,2}$/ } keys %$href) {
+                        die "not a valid number\n";
+                    }
+                    elsif (any { $href->{$_} < 0 || $href->{$_} > 23 } keys %$href) {
+                        die "hour out of range\n";
+                    }
+                    else {
+                        return true;
+                    }
+                }
+            },
         },
         datetime => {
             type => OBJECT,
@@ -171,6 +194,8 @@ sub parse_datetime
 
     $self->{input_string} = $self->{date_string};
 
+    $self->{mode} = 'parse';
+
     my $date_string = $self->{date_string};
 
     $self->_rewrite(\$date_string);
@@ -216,6 +241,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
@@ -252,6 +279,8 @@ sub parse_datetime
 
         $self->_set(%args);
 
+        $self->{datetime}->truncate(to => 'second');
+        $self->_set_truncated;
         $self->_set_valid_exp;
     }
     else {
@@ -304,7 +333,7 @@ sub _parse_init
             $self->{datetime} = dclone($self->{Datetime});
         }
         else {
-            $self->{datetime} = DateTime->$method(
+            $self->{datetime} = DateTime::HiRes->$method(
                 time_zone => $self->{Time_zone},
                 %$args,
             );
@@ -324,6 +353,7 @@ sub _parse_init
     $self->_unset_error;
     $self->_unset_valid_exp;
     $self->_unset_trace;
+    $self->_unset_truncated;
 }
 
 sub parse_datetime_duration
@@ -349,10 +379,12 @@ sub parse_datetime_duration
         $shrinked = true;
     }
 
-    $self->_pre_duration(\@date_strings);
-    $self->{state} = {};
+    $self->_rewrite_duration(\@date_strings);
 
-    my (@queue, @traces);
+    $self->_pre_duration(\@date_strings);
+    @$self{qw(state truncated_duration)} = ({}, []);
+
+    my (@queue, @traces, @truncated);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
         $self->_save_state(
@@ -363,15 +395,19 @@ sub parse_datetime_duration
         if (@{$self->{traces}}) {
             push @traces, $self->{traces}[0];
         }
+        if ($self->{running_tests}) {
+            push @truncated, $self->_get_truncated;
+        }
     }
 
-    $self->_post_duration(\@queue, \@traces);
+    $self->_post_duration(\@queue, \@traces, \@truncated);
     $self->_restore_state;
 
     delete @$self{qw(duration insert state)};
 
-    @{$self->{traces}} = @traces;
-    $self->{input_string} = $duration_string;
+    @{$self->{traces}}             = @traces;
+    @{$self->{truncated_duration}} = @truncated;
+    $self->{input_string}          = $duration_string;
 
     if ($shrinked) {
         $self->_set_failure;
@@ -388,8 +424,18 @@ sub extract_datetime
     my $extract_string;
     $self->_params_init(@_, { string => \$extract_string });
 
+    $self->_unset_failure;
+    $self->_unset_error;
+    $self->_unset_valid_exp;
+
+    $self->{input_string} = $extract_string;
+
+    $self->{mode} = 'extract';
+
     my @expressions = $self->_extract_expressions($extract_string);
 
+    $self->_set_valid_exp if @expressions;
+
     return wantarray ? @expressions : $expressions[0];
 }
 
@@ -406,8 +452,22 @@ sub error
 
     return '' if $self->success;
 
-    my $error  = "'$self->{input_string}' does not parse ";
-       $error .= $self->_get_error || '(perhaps you have some garbage?)';
+    my $error = sub
+    {
+        return undef unless defined $self->{mode} && length $self->{mode};
+        my %errors = (
+            extract => "'$self->{input_string}' cannot be extracted from",
+            parse   => "'$self->{input_string}' does not parse",
+        );
+        return $errors{$self->{mode}};
+    }->();
+
+    if (defined $error) {
+        $error .= ' ' . ($self->_get_error || '(perhaps you have some garbage?)');
+    }
+    else {
+        $error = 'neither extracting nor parsing method invoked';
+    }
 
     return $error;
 }
@@ -416,7 +476,7 @@ sub trace
 {
     my $self = shift;
 
-    return @{$self->{traces}};
+    return @{$self->{traces} || []};
 }
 
 sub _process
@@ -534,6 +594,7 @@ sub _truncate
         my $index = $indexes{$unit} - 1;
         if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
             $self->{datetime}->truncate(to => $unit);
+            $self->_set_truncated;
             last;
         }
     }
@@ -573,13 +634,13 @@ sub _advance_future
 
     my $now = exists $self->{Datetime}
       ? dclone($self->{Datetime})
-      : DateTime->now(time_zone => $self->{Time_zone});
+      : DateTime::HiRes->now(time_zone => $self->{Time_zone});
 
     my $day_of_week = sub { $_[0]->_Day_of_Week(map $_[0]->{datetime}->$_, qw(year month day)) };
 
     my $skip_weekdays = false;
 
-    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+    if ((all { /^(?:(?:nano)?second|minute|hour)$/ } keys %modified)
         && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
         && (($self->{Prefer_future} && $self->{datetime} <  $now)
          || ($self->{Demand_future} && $self->{datetime} <= $now))
@@ -651,18 +712,23 @@ sub _get_valid_exp   { $_[0]->{valid_expression}         }
 sub _set_valid_exp   { $_[0]->{valid_expression} = true  }
 sub _unset_valid_exp { $_[0]->{valid_expression} = false }
 
+sub _get_truncated   { $_[0]->{truncated}         }
+sub _set_truncated   { $_[0]->{truncated} = true  }
+sub _unset_truncated { $_[0]->{truncated} = false }
+
 sub _get_datetime_object
 {
     my $self = shift;
 
     my $dt = DateTime->new(
-        time_zone => $self->{datetime}->time_zone,
-        year      => $self->{datetime}->year,
-        month     => $self->{datetime}->month,
-        day       => $self->{datetime}->day_of_month,
-        hour      => $self->{datetime}->hour,
-        minute    => $self->{datetime}->minute,
-        second    => $self->{datetime}->second,
+        time_zone  => $self->{datetime}->time_zone,
+        year       => $self->{datetime}->year,
+        month      => $self->{datetime}->month,
+        day        => $self->{datetime}->day_of_month,
+        hour       => $self->{datetime}->hour,
+        minute     => $self->{datetime}->minute,
+        second     => $self->{datetime}->second,
+        nanosecond => $self->{datetime}->nanosecond,
     );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
@@ -809,7 +875,7 @@ recognized by L<DateTime>. Defaults to 'floating'.
 
 =item * C<daytime>
 
-An anonymous hash reference consisting of customized daytime hours,
+A hash reference consisting of customized daytime hours,
 which may be selectively changed.
 
 =back
@@ -943,6 +1009,9 @@ valuable suggestions and patches:
  Ricardo Signes
  Felix Ostmann
  Jrn Clausen
+ Jim Avera
+ Olaf Alders
+ Karen Etheridge
 
 =head1 SEE ALSO
 
